@@ -1,3 +1,5 @@
 from genz_tokenize.tokenize import Tokenize
 from genz_tokenize.tokenize import TokenizeForBert
 from genz_tokenize.tokenize import get_embedding_matrix
+from genz_tokenize import preprocess
+from genz_tokenize import models
